
# q-expansion of newform 1078.4.a.e, downloaded from the LMFDB on 01 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1078
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[2], [-7], [-3], [0], [-11], [16], [-6], [-14], [-51], [54], [-95], [-193], [-102], [284], [72], [-102], [63], [790], [-433], [135], [238], [770], [1008], [639], [-11], [-1692], [532], [1350], [1616], [597], [302], [462], [3021], [1594], [2814], [2450], [-3899], [-124], [30], [-2268], [1731], [-785], [51], [1226], [2754], [1744], [-4210], [5497], [3732], [3883], [-222], [-2304], [1276], [255], [-2370], [2466], [-6774], [4876], [-3268], [5928], [-2672], [-5322], [-2816], [-8760], [-1337], [4269], [-4843], [-3790], [1722], [3166], [4167], [1380], [-755], [-8596], [-11287], [1185], [-11643], [3382], [-7566], [-86], [12672], [12398], [12372], [14929], [-9146], [-11253], [6711], [16952], [5850], [-4957], [-3753], [16272], [10613], [11562], [-9604], [15852], [-15669], [2817], [12364], [2372], [-12832], [-14166], [18540], [-21894], [23204], [-18479], [1116], [3654], [10248], [-18848], [13198], [1712], [20994], [-25691], [25445], [19095], [24391], [5691], [-25749], [-9030], [-28895], [16856], [19470], [-14820], [18451], [6288], [-17629], [-2067], [-11315], [23644], [-862], [9072], [9407], [-15946], [8220], [-14372], [-14082], [12742], [-7131], [21966], [10978], [-27534], [-19591], [12744], [-23567], [-30219], [-8654], [43284], [21931], [-22404], [4826], [-40203], [-41716], [44358], [29384], [5160], [10436], [-40626], [2608], [57150], [-657], [5508], [-1690], [-7263], [1569], [9261], [19400], [20740], [8960], [-29313], [18636], [41254], [31482], [49316], [33415], [45105], [-55438], [58443], [6010], [-40466], [51884], [55224], [-33847], [-19404], [-7353], [9618], [33212], [13060], [-36376], [-5058], [-37403], [-29022], [-54385], [22806], [31194], [-8277], [-4336], [25787], [-45486], [-29826], [33930], [-49868], [-24446], [-32330], [-73815], [68709], [52423], [-60174], [17844], [-8228], [76964], [-18753], [-22111], [-36924], [15360], [-61817], [-84544], [26910], [66684], [33723], [-75460], [-20666], [-74211], [80147], [-38724], [48890], [38058], [44856], [-41108], [-37788], [-29839], [-86114], [-63592], [-15540], [97279], [92946], [-89915], [35958], [62829], [-92181], [81357], [-50066], [-12476], [-84175], [83622], [4830], [-80840], [-30492], [98606], [-35358], [-37984], [-68856], [-60969], [18154], [52716], [-124290], [11246], [-71738], [110973], [49444], [812], [-3072], [64486], [-74768], [130722], [-107156], [27435], [-18435], [-98782], [74160], [91381], [-21139], [-103358], [-21388], [78766], [-39107], [137001], [4202], [73154], [-131040], [114636], [50339], [36126], [-85658], [32002], [142431], [7658], [-46938], [-2069], [-70056], [-69771], [89436], [24348], [76260], [12638], [-15336], [8995], [111039], [68010], [-145976], [-83360], [-122772], [-136474], [-101844], [-156907], [56183], [-5643], [-8615], [79944], [-52564], [47922], [-69534], [76554], [-120841], [33882], [-184742], [108105], [-52854], [-58598], [-47154], [79996], [35117], [-163818], [9119], [92106], [127915], [43733], [7933], [3438], [86400], [35000], [-147666], [-21062], [-108486], [192854], [137823], [-120679], [-39630], [50455], [-13604], [31421], [144831], [-75588], [88805], [65691], [208824], [63343], [71489], [-75384], [203955], [30472], [123455], [-110070], [-4322], [103004], [102252], [-23445], [25542], [148248], [-53889], [88454], [164106], [223497], [174336], [48889], [-47563], [30744], [-68308], [90956], [202311], [171220], [-68412], [189408], [171808], [-144427], [-140748], [-85638], [-6014], [-38256], [-114752], [62526], [111687], [-54430], [-15966], [53260], [222609], [103520], [-119711], [-178810], [261381], [83153], [87519], [-58233], [-22955], [-262191], [-51946], [-140888], [174366], [-171364], [62880], [171574], [135309], [246440], [-115866], [-259560], [79510], [34922], [258696], [267556], [-255459], [-85160], [-31848], [113220], [-138328], [231254], [-23535], [43872], [93721], [-16650], [-61698], [39375], [-114776], [-4236], [110910], [124810], [164175], [10113], [112464], [173149], [117126]]
